@@ -5,8 +5,8 @@
 
 #include <sys/types.h>
 
-void get_ibm_acpi_fan(struct text_object *, char *, int);
-void get_ibm_acpi_temps(void);
+int get_ibm_acpi_fan(struct text_object *, char *, int);
+int get_ibm_acpi_temps(void);
 void get_ibm_acpi_volume(struct text_object *, char *, int);
 void get_ibm_acpi_brightness(struct text_object *, char *, int);
 
