@@ -210,6 +210,8 @@ static int process_parse_stat(struct process *process)
 	int nice_val;
 	char *lparen, *rparen;
 
+	struct stat process_stat;
+
 	snprintf(filename, sizeof(filename), PROCFS_TEMPLATE, process->pid);
 
 	ps = open(filename, O_RDONLY);
@@ -218,6 +220,11 @@ static int process_parse_stat(struct process *process)
 		return 1;
 	}
 
+	if (fstat(ps, &process_stat) != 0){
+		return 1;
+	}
+	process->uid=process_stat.st_uid;
+
 	/* Mark process as up-to-date. */
 	process->time_stamp = g_time;
 
@@ -237,7 +244,7 @@ static int process_parse_stat(struct process *process)
 	strncpy(procname, lparen + 1, rc);
 	procname[rc] = '\0';
 	rc = sscanf(rparen + 1, "%3s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %lu "
-			"%lu %*s %*s %*s %d %*s %*s %*s %u %u", state, &process->user_time,
+			"%lu %*s %*s %*s %d %*s %*s %*s %llu %llu", state, &process->user_time,
 			&process->kernel_time, &nice_val, &process->vsize, &process->rss);
 	if (rc < 6) {
 		NORM_ERR("scaning data for %s failed, got only %d fields", procname, rc);
@@ -864,6 +871,10 @@ int parse_top_args(const char *s, const char *arg, struct text_object *obj)
 			td->type = TOP_MEM_RES;
 		} else if (strcmp(buf, "mem_vsize") == EQUAL) {
 			td->type = TOP_MEM_VSIZE;
+		} else if (strcmp(buf, "uid") == EQUAL) {
+			td->type = TOP_UID;
+		} else if (strcmp(buf, "user") == EQUAL) {
+			td->type = TOP_USER;
 #ifdef IOSTATS
 		} else if (strcmp(buf, "io_read") == EQUAL) {
 			td->type = TOP_READ_BYTES;
@@ -875,10 +886,10 @@ int parse_top_args(const char *s, const char *arg, struct text_object *obj)
 		} else {
 			NORM_ERR("invalid type arg for top");
 #ifdef IOSTATS
-			NORM_ERR("must be one of: name, cpu, pid, mem, time, mem_res, mem_vsize, "
+			NORM_ERR("must be one of: name, cpu, pid, mem, time, mem_res, mem_vsize, uid, user"
 					"io_read, io_write, io_perc");
 #else /* IOSTATS */
-			NORM_ERR("must be one of: name, cpu, pid, mem, time, mem_res, mem_vsize");
+			NORM_ERR("must be one of: name, cpu, pid, mem, time, mem_res, mem_vsize, uid, user");
 #endif /* IOSTATS */
 			free(td->s);
 			free(obj->data.opaque);
@@ -986,9 +997,11 @@ void print_top(struct text_object *obj, char *p, int p_max_size)
 
 		switch (td->type) {
 			case TOP_NAME:
-				width = MIN(p_max_size, (int)top_name_width + 1);
-				snprintf(p, width + 1, "%-*s", width,
-						needed[td->num]->name);
+				if (needed[td->num]->name) {
+					width = MIN(p_max_size, (int)top_name_width + 1);
+					snprintf(p, width + 1, "%-*s", width,
+							needed[td->num]->name);
+				}
 				break;
 			case TOP_CPU:
 				width = MIN(p_max_size, 7);
@@ -1022,6 +1035,14 @@ void print_top(struct text_object *obj, char *p, int p_max_size)
 				human_readable(needed[td->num]->vsize,
 						p, p_max_size);
 				break;
+			case TOP_UID:
+				snprintf(p, p_max_size, "%5d", needed[td->num]->uid);
+				break;
+			case TOP_USER:
+				width = MIN(p_max_size, 9);
+				snprintf(p, width, "%.8s",
+						getpwuid(needed[td->num]->uid)->pw_name);
+				break;
 #ifdef IOSTATS
 			case TOP_READ_BYTES:
 				human_readable(needed[td->num]->read_bytes / update_interval,
